#include "StdAfx.h"
#include "GameTalkAPIGlobal.h"
#include "ProfileImageActor.h"
#include "NpProtocolCodeDefine.h"
#include "MessengerActor.h"
#include "UtilFunction.h"
#include "GameTalkNotifyCommand.h"
#include "ConvertUTF.h"
#include "GTCommonAPI.h"
#include <fcntl.h>
#include "ProfileImageRecverP2P.h"

#define POLYNOMIAL				0x04c11db7L
#define SEND_FRAME_SIZE         4096

static unsigned long crc_table[256];


CProfileImageActor::CProfileImageActor(CMessengerActor *pActor) 
    : CActor(pActor)
	, m_bUploadingProfileImage(false)
	, m_tridSetProfileImage()
	, m_pMessengerActor(pActor)
	, m_pProfileImageProxy(NULL)
#ifdef __USE_PROFILE_IMG_BY_P2P__
	, m_pNpP2PMngr(NULL)
#endif
{
	//CGTLifeLog log(__FUNCTION__);
    
	register int i, j;
	register unsigned long crc_accum;
    
	for(i = 0; i < 256; i++)
	{
		crc_accum = ((unsigned long) i << 24);
		for(j = 0; j < 8; j++)
		{
			if(crc_accum & 0x80000000L)
				crc_accum = (crc_accum << 1) ^ POLYNOMIAL;
			else
				crc_accum = (crc_accum << 1);
		}
		crc_table[i] = crc_accum;
	}
    
	CWASActor *pWASActor = static_cast<CWASActor *>(m_pMessengerActor->GetWASActor());
	if(pWASActor)
		pWASActor->RegisterObserverEx(this);
}


CProfileImageActor::~CProfileImageActor(void)
{
	//CGTLifeLog log(__FUNCTION__);
    
	CWASActor *pWASActor = static_cast<CWASActor *>(m_pMessengerActor->GetWASActor());
	if(pWASActor)
		pWASActor->UnRegisterObserverEx(this);
    
//	log.Trace("%s : DELETE ACTOR RESULT => OBSERVER = %d, REF = %d", __FUNCTION__, size(), RetainCount());
//	log.End();
}


// static function 
WSTRING CProfileImageActor::GetProfileImageFilePath(const WSTRING &strUserID, const WSTRING &strFileName)
{
	WSTRING strResult;
	WSTRING strPath;
	strPath = GT_GLOBAL->GetEnvironment().strProfileImagePath + strUserID;
    
	FileUtil::ForceDirectory(strPath.c_str());
    
	strResult = strPath + STR_PATH_DELIMITER;
    
	strResult+= strFileName;
    
	return strResult;
}


// static function 
DWORD CProfileImageActor::GetFileSize(LPCTSTR pszFileName, LPDWORD lpFileSizeHigh)
{
	DWORD dwRet = 0;
#if defined(WIN32)	// __KEH_USE_PROFILE_IMAGE__
	if(lpFileSizeHigh)
		*lpFileSizeHigh = 0;
    
	HANDLE hFile = CreateFile(pszFileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if(hFile == INVALID_HANDLE_VALUE)
		return 0;
    
	dwRet = ::GetFileSize(hFile, lpFileSizeHigh);
	CloseHandle(hFile);
#else
	USES_UTF8_CONVERSION;
	dwRet = GTCommonAPI::GetFileSize(W2U8(pszFileName));
#endif	
	return dwRet;
}

// static function 
DWORD CProfileImageActor::GetFileCRC(LPCTSTR pszFileName, DWORD * pdwCRC)
{
	DWORD dwFileSize = 0;
	DWORD dwCRC = 0;
	DWORD dwRead, dwTotalRead = 0;
    
	char buf[SEND_FRAME_SIZE];
    
	if(pdwCRC == NULL)
		return FALSE;
#if defined(WIN32)	// __KEH_USE_PROFILE_IMAGE__
	HANDLE hFile = CreateFile(pszFileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if(hFile == INVALID_HANDLE_VALUE)
		return 0;
    
	dwFileSize = ::GetFileSize(hFile, NULL);
	while(dwTotalRead < dwFileSize)
	{
		if(ReadFile(hFile, buf, SEND_FRAME_SIZE, &dwRead, NULL) == FALSE)
		{
			CloseHandle(hFile);
			return 0;
		}
		dwTotalRead += dwRead;
		dwCRC = GetCRC(dwCRC, buf, dwRead);
	}
	CloseHandle(hFile);
#else
	USES_UTF8_CONVERSION;
    
	const char *szFileName = W2U8(pszFileName);
    
	int fd = open(szFileName, O_RDONLY);
	
	if(-1 == fd)
		return 0;
	
	dwFileSize = GTCommonAPI::GetFileSize(szFileName);
    
	while(dwTotalRead < dwFileSize)
	{
		dwRead = read(fd, buf, sizeof(buf));
		
		dwCRC = GetCRC(dwCRC, buf, dwRead);
		
		dwTotalRead += dwRead;
	}
	close(fd);
#endif	
    
	if(pdwCRC)
		*pdwCRC = dwCRC;
    
	return dwFileSize;
}

// static function 
/* update the CRC on the data block one byte at a time */
DWORD CProfileImageActor::GetCRC(DWORD nCRC_Accum, char * pData,int nSize)
{
	register int i, j;
    
	for(j=0; j<nSize; j++)
	{
		i = ((int)(nCRC_Accum >> 24) ^ *pData++) & 0xff;
		nCRC_Accum = (nCRC_Accum << 8) ^ crc_table[i];
	}
	return nCRC_Accum;
}

/* static */
WSTRING CProfileImageActor::GetProfileImageFilePathBySizeType(LPCWSTR strUserID, GTProfileImageSizeType eSizeType)
{
    WSTRING strFileName;
    switch(eSizeType)
    {
        case profileOriginalSize:
            strFileName = _T("PF_ORG.GT");
            break;
        case profileThumbLargeSize:
            strFileName = _T("PF_LARGE.GT");
            break;
        case profileThumbSmallSize:
            strFileName = _T("PF_SMALL.GT");
            break;
    }
    
    return GetProfileImageFilePath(strUserID, strFileName);
}


void CProfileImageActor::_OnSuccessGetUserImageOK(LPCWSTR strUserID, LPCWSTR strFilePath)
{
	//if(StringUtil::StrCompare(m_pMessengerActor->GetUserID(), strUserID) == 0)
	//{
	//	m_strMyProfileImageFileName = strFilePath;
	//}
}




GTProfileImageCheckResult CProfileImageActor::_GetUserProfileImagePathEx(LPCWSTR strUserID, GTProfileImageSizeType eSizeType, DWORD &dwCRC/*OUT*/, size_t &nBufferSize/*OUT*/, char *szFilePath /*OUT*/)
{
    WSTRING strToFilePath;
    strToFilePath = GetProfileImageFilePathBySizeType(strUserID, eSizeType);    
    
    GTProfileImageCheckResult result = _CheckImage(strUserID, strToFilePath.c_str(), eSizeType, dwCRC);
    switch(result)
    {
        case GTProfileImgCheckFileNotExist:
        case GTProfileImgCheckEmptyImage:
            strToFilePath = _T("");
        default:
            break;
    }

	USES_UTF8_CONVERSION;
        
    // SAVE [OUTPUT]
    if(szFilePath && StringUtil::StrLen(strToFilePath.c_str()) > 0)
    {
        _snprintf(szFilePath, nBufferSize-1, "%s", W2U8(strToFilePath.c_str()));
        nBufferSize = strlen(szFilePath);
    }
	else
	{
		nBufferSize = 0;

		if(GT_GLOBAL->GetEnvironment().strDefaultProfileImageFile.length() > 0)
		{
			 _snprintf(szFilePath, nBufferSize-1, "%s", W2U8(GT_GLOBAL->GetEnvironment().strDefaultProfileImageFile.c_str()));
			 nBufferSize = strlen(szFilePath);
		}
	}
    
    return result;
}


GTProfileImageCheckResult CProfileImageActor::GetUserProfileImagePath(LPCWSTR strUserID, GTProfileImageSizeType eSizeType, size_t &nBufferSize/*OUT*/, char *szFilePath /*OUT*/)
{
	DWORD dwCRCTemp = 0;
    return _GetUserProfileImagePathEx(strUserID, eSizeType, dwCRCTemp, nBufferSize, szFilePath);
}


#ifdef __USE_PROFILE_IMG_BY_P2P__
bool CProfileImageActor::RequestProfileImageByP2P(LPCWSTR strUserID, GTProfileImageSizeType eSizeType, DWORD dwUserData1, DWORD dwUserData2, bool bForceMode, trid_t *pTrid)
{
	if(StringUtil::StrCompare(strUserID, GetMessengerActor()->GetUserID()) == 0)
	{
		return RequestProfileImage(strUserID, eSizeType, dwUserData1, dwUserData2, bForceMode, pTrid);
	}

	CProfileImageRecverP2P *pRecver = NULL;
	profileimgrecvermap_t::iterator it = m_mapProfileImageRecver.find(strUserID);
	if(it == m_mapProfileImageRecver.end())
	{
		pRecver = new CProfileImageRecverP2P(this);
		if (NULL == pRecver)
			return false;

		m_mapProfileImageRecver.insert(profileimgrecvermap_t::value_type(strUserID, pRecver));
	}
	else
	{
		return false;
	}

	if(pRecver->RequestGetImage(strUserID, eSizeType, dwUserData1, dwUserData2) == false)
	{
		DeleteRecver(strUserID);
		return false;
	}

	return true;
}
#endif


bool CProfileImageActor::RequestProfileImage(LPCWSTR strUserID, GTProfileImageSizeType eSizeType, DWORD dwUserData1, DWORD dwUserData2, bool bForceMode, trid_t *pTrid)
{
	//CGTLifeLog log(__FUNCTION__);
    
    char szFilePath[1024] = {0, };
    size_t nSize = sizeof(szFilePath);
    
    if(bForceMode == false)
    {
		DWORD dwCRC = 0;
        GTProfileImageCheckResult result = _GetUserProfileImagePathEx(strUserID, eSizeType, dwCRC, nSize, szFilePath); 
        if(result == GTProfileImgCheckSuccess || result == GTProfileImgCheckEmptyImage)
        {
            USES_UTF8_CONVERSION;
            P_GT_PROFILE_IMAGE_RESULT_DATA sResultData;        
            sResultData.trid = -1;
            sResultData.dwProfileImageCRC = dwCRC;
            sResultData.eProfileError = piError_None;
            sResultData.eWasError = wasError_None;
            sResultData.strFilePath = U82W(szFilePath);
            sResultData.strUserID = strUserID;
            sResultData.bSuccess = true;
            sResultData.eSizeType = eSizeType;

			_OnSuccessGetUserImageOK(strUserID, sResultData.strFilePath.c_str());
            CProfileImageResponseNotifyCommand *pNotify = CProfileImageResponseNotifyCommand::Create(this, sResultData, dwUserData1, dwUserData2, true);
            
            if(pTrid)
            {
                *pTrid = sResultData.trid;
            }

            if(pNotify)
                pNotify->ExecuteAsync();            
            return true;
        }
    }
    
    WSTRING strFilePath = GetProfileImageFilePathBySizeType(strUserID, eSizeType);
    
    GT_PROFILE_IMAGE_DATA sData;
    sData.strUserID = strUserID;
    sData.strFileName = strFilePath.c_str();
    sData.dwCRC = 0;
    sData.eSizeType = eSizeType;
    
    trid_t tridDown;
	CWASActor *pWASActor = static_cast<CWASActor *>(GetMessengerActor()->GetWASActor());
	if(pWASActor)
    {
        if(pWASActor->RequestProfileImageProfile(false, sData, dwUserData1, dwUserData2, &tridDown))
        {
            if(pTrid)
            {
                *pTrid = tridDown;
            }
            _AddTransaction(tridDown);
            return true;
        }
    }
    
    return false;
}



bool CProfileImageActor::RequestSetMyProfileImage(LPCWSTR pszFileName, trid_t *pTrid)
{
	//CGTLifeLog log(__FUNCTION__);
    
	if(m_bUploadingProfileImage)
		return FALSE;
    
	trid_t trid = GT_GLOBAL->GetTransactionID(pTrid);
	m_tridSetProfileImage = trid;
    
	GT_PROFILE_IMAGE_DATA data;
	data.strUserID = m_pMessengerActor->GetUserID();
	data.strFileName = pszFileName;
    
	bool bRet = false;
	
	CWASActor *pWASActor = static_cast<CWASActor *>(m_pMessengerActor->GetWASActor());
	if(pWASActor)
		bRet = pWASActor->RequestProfileImageProfile(true, data, 0, 0, &trid);
    
	if(bRet)
	{
		_AddTransaction(trid);
		m_bUploadingProfileImage = true;
	}
    
	return bRet;
}


void CProfileImageActor::StopSetMyProfileImage(void)
{
	//CGTLifeLog log(__FUNCTION__);
    
	CWASActor *pWASActor = static_cast<CWASActor *>(m_pMessengerActor->GetWASActor());
	if(pWASActor)
		pWASActor->CancelCommand(m_tridSetProfileImage);
}


GTProfileImageCheckResult CProfileImageActor::_CheckImage(LPCWSTR strUserID, LPCWSTR strFilePath, GTProfileImageSizeType eSizeType, DWORD &dwCRC)
{
    USES_UTF8_CONVERSION;
    
    GTProfileImageCheckResult internalResult;
    if(m_pProfileImageProxy)
    {
		dwCRC = 0;

        unsigned int nTempCRC = 0;
        if (IsReusableProfileImage(strUserID, eSizeType, &nTempCRC)) 
        {
            dwCRC = nTempCRC;
            
            if(dwCRC == 0)
                return GTProfileImgCheckEmptyImage;
            
            internalResult = GTProfileImgCheckSuccess;
        }
        else
            internalResult = GTProfileImgCheckCanNotReusable;
    }
    
    if(strFilePath && StringUtil::StrLen(strFilePath) > 0)
    {
        if(GTCommonAPI::IsFileExist(W2U8(strFilePath)) == false)
        {
            return GTProfileImgCheckFileNotExist;
        }
    }
    
	return internalResult;
}


bool CProfileImageActor::_OnResponseSuccUploadImage(const GT_PROFILE_IMAGE_DATA& sData)
{
	m_bUploadingProfileImage = false;
	
	_OnSuccessGetUserImageOK(sData.strUserID, sData.strFileName);

    // That image may be optimized on remote server and automatically thumbnail image created from original image.
    // When a specific profile image is needed, you just request to download with associated profile image size type. 
    // with selecting type - original, small-thumb, large-thumb -
    
    if(StringUtil::StrLen(sData.strFileName) > 0)
    {
        WSTRING strFilePath = GetProfileImageFilePathBySizeType(m_pMessengerActor->GetUserID(), profileOriginalSize);

        nfc::CUTF8Buffer utf1, utf2;

        LPCSTR utf8_from = utf1.Encode(sData.strFileName);
        LPCSTR utf8_to = utf2.Encode(strFilePath.c_str());
        GTCommonAPI::CopyFile(utf8_from, utf8_to);
    }
    
    m_pMessengerActor->SetProfileImageData(sData);
	
    SaveCurrentProfileImageCRC(sData.strUserID, sData.dwCRC, profileOriginalSize); 
//  UpdateProfileImageCRC(sData.strUserID, sData.dwCRC);
    
    return true;
}



bool CProfileImageActor::_OnResponseSuccDownloadImage(const GT_PROFILE_IMAGE_DATA& sData)
{
    // TO DO..
	_OnSuccessGetUserImageOK(sData.strUserID, sData.strFileName);

	return true;
}


bool CProfileImageActor::IsReusableProfileImage(LPCWSTR strUserId, GTProfileImageSizeType eSizeType, unsigned int *pnCRC)
{
    if(NULL == m_pProfileImageProxy)
        return false;
    
    if(pnCRC)
    {
         unsigned int currentCRC = m_pProfileImageProxy->GetCurrentImageCRC(strUserId);
        
        if(currentCRC > 0)
            *pnCRC = currentCRC;
        else
            *pnCRC = 0;
    }
    
    GTProfileImageSizeType recvedBits = m_pProfileImageProxy->GetImageRecvedTypeBits(strUserId);
    
    if((recvedBits & eSizeType) != 0)
        return true;
    
    return false;
}

bool CProfileImageActor::UpdateProfileImageCRC(LPCWSTR strUserId, DWORD dwProfileImgCrc)
{
    if(NULL == m_pProfileImageProxy)
        return false;
    
    unsigned int previousCRC = m_pProfileImageProxy->GetCurrentImageCRC(strUserId);
        
    if(previousCRC != dwProfileImgCrc)
    {
        m_pProfileImageProxy->SetImageRecvedTypeBits(strUserId, 0);
    }
    return true;
}


bool CProfileImageActor::SaveCurrentProfileImageCRC(LPCWSTR strUserId, DWORD dwProfileImgCrc, GTProfileImageSizeType eSizeType)
{
    if(NULL == m_pProfileImageProxy)
        return false;
    
    unsigned int previousCRC = m_pProfileImageProxy->GetCurrentImageCRC(strUserId);
    
    m_pProfileImageProxy->SetCurrentImageCRC(strUserId, dwProfileImgCrc);
    
    GTProfileImageSizeType updateBits = 0;
    
    if(dwProfileImgCrc != 0)    
    {
        if(previousCRC == 0)
        {
            // first.. just save..
            updateBits = eSizeType;
        }
        else
        {
            if(previousCRC != dwProfileImgCrc)
            {
                // new.. clear
                updateBits = eSizeType;
            }
            else
            {
                // same crc.. flag bits update..
                updateBits = m_pProfileImageProxy->GetImageRecvedTypeBits(strUserId);
                updateBits |= eSizeType;
            }
        }
    }
    else// MEAN EMPTY IMAGE
    {
        // SET EMPTY IMAGE..
        // SO ALL IMAGE - RECVED MASK -
        updateBits = 0xFFFF;
    }

    m_pProfileImageProxy->SetImageRecvedTypeBits(strUserId, updateBits);
    
    return true;
}


#ifdef __USE_PROFILE_IMG_BY_P2P__

void CProfileImageActor::SetOFFLINE()
{
	profileimgsendermap_t::iterator it2 = m_mapProfileImageSender.begin();
	for(; it2 != m_mapProfileImageSender.end(); it2++)
	{
		delete it2->second;
	}
	m_mapProfileImageSender.clear();

	profileimgrecvermap_t::iterator it3 = m_mapProfileImageRecver.begin();
	for(; it3 != m_mapProfileImageRecver.end(); it3++)
	{
		delete it3->second;
	}
	m_mapProfileImageRecver.clear();

	m_pNpP2PMngr->DeregisterHandler(REQ_PFIMG_CKSUM);
	m_pNpP2PMngr->DeregisterHandler(REQ_PFIMG_REQDN);
	m_pNpP2PMngr = NULL;
}


void CProfileImageActor::Delete()
{
	// Nothing to do..
}


void CProfileImageActor::DeleteRecver(LPCWSTR strUserID)
{
	if(strUserID == NULL)
		return;

	profileimgrecvermap_t::iterator it = m_mapProfileImageRecver.find(strUserID);
	if(it == m_mapProfileImageRecver.end())
		return;

	delete it->second;

	m_mapProfileImageRecver.erase(it);
}


void CProfileImageActor::DeleteSender(CProfileImageSender *pSender)
{
	_EraseProfileImageSender(pSender);
}


CProfileImageSender* CProfileImageActor::_CreateProfileImageSender()
{
	CProfileImageSender * pSender = new CProfileImageSender(this);
	if(pSender == NULL)
		return NULL;

	m_mapProfileImageSender.insert(profileimgsendermap_t::value_type(pSender, pSender));

	return pSender;
}


void CProfileImageActor::_EraseProfileImageSender(CProfileImageSender *pSender)
{
	if(pSender == NULL)
		return;

	profileimgsendermap_t::iterator it = m_mapProfileImageSender.find(pSender);
	if(it == m_mapProfileImageSender.end())
		return;

	m_mapProfileImageSender.erase(it);
	delete pSender;
}

//------------------------------------------------------------------------------------------------
//
// _INpP2PMngr_IncomingP2PSessionHandler 
//
//------------------------------------------------------------------------------------------------

bool CALLBACK CProfileImageActor::NpP2PMngr_OnIncoming_NpStreamSession(INpStreamSession * pIncomingSession, 
                                                                         LPCWSTR wszFromServiceId,
                                                                         INpPacket * pPacket,
                                                                         NPP2PSESSION_PROTOCOL_TYPE ePType,
                                                                         bool bIsRelay,
                                                                         bool bIsDual)
{
	CProfileImageSender * pSender = _CreateProfileImageSender();
	if(pSender == NULL)
		return false;

	pSender->SetStreamSession(pIncomingSession, pPacket);

	// ݵ true ؾ.
	// DLL ο false Ͻ pIncomingSession  .
	return true;
}
#endif


#ifdef __USE_PROFILE_IMG_BY_P2P__
//------------------------------------------------------------------------------------------------
//
// ProfileImage ̺Ʈ 
//
//------------------------------------------------------------------------------------------------


void CProfileImageActor::OnProfileImageError(CProfileImageRecverP2P *pImage, GTWASCallErrorCode eWASError, GTProfileImgError ePIError)
{
	if(pImage == NULL)													   
		return;

	P_GT_PROFILE_IMAGE_DATA sData;
	pImage->GetData(sData);

	P_GT_PROFILE_IMAGE_RESULT_DATA sResultData;
	sResultData.eSizeType = sData.eSizeType;
	sResultData.trid = -1;
	sResultData.eProfileError = ePIError;
	sResultData.eWasError = eWASError;
	sResultData.strUserID = pImage->GetUserID();
	sResultData.bSuccess = false;

	//  FIRE!
	CProfileImageResponseNotifyCommand *pNotify = CProfileImageResponseNotifyCommand::Create(this, sResultData, pImage->GetUserData1(), pImage->GetUserData2(), false);
	if(pNotify)
		pNotify->ExecuteAsync();

	DeleteRecver(pImage->GetUserID());
}


void CProfileImageActor::OnProfileImageReady(CProfileImageRecverP2P *pImage)
{
	if(pImage == NULL)													   
		return;

	P_GT_PROFILE_IMAGE_DATA sData;
	pImage->GetData(sData);

	P_GT_PROFILE_IMAGE_RESULT_DATA sResultData;
	sResultData.trid = -1;
	sResultData.eSizeType = sData.eSizeType;
	sResultData.eProfileError = piError_None;
	sResultData.eWasError = wasError_None;
	sResultData.strUserID = pImage->GetUserID();
	sResultData.strFilePath = sData.strFileName;

	if((sResultData.eWasError == wasError_None) && 
	   (sResultData.eProfileError == piError_None) )
	   sResultData.bSuccess = true;
	else
	   sResultData.bSuccess = false;

	_OnSuccessGetUserImageOK(sResultData.strUserID.c_str(), sResultData.strFilePath.c_str());

	//  FIRE!
	CProfileImageResponseNotifyCommand *pNotify = CProfileImageResponseNotifyCommand::Create(this, sResultData, pImage->GetUserData1(), pImage->GetUserData2(), false);
	if(pNotify)
		pNotify->ExecuteAsync();

	DeleteRecver(pImage->GetUserID());
}
#endif


//------------------------------------------------------------------------------------------------
//
// IGTWASActorEvent 
//
//------------------------------------------------------------------------------------------------

bool CProfileImageActor::OnGTWASActor_ResponseProfileImage(IGTWASActor *pActor, 
														   trid_t trid,
														   GTWASCallErrorCode eErrorCode, 
														   GTProfileImgError eProfileError, 
														   bool bUploadFile, 
														   const GT_PROFILE_IMAGE_DATA& sData,
                                                           DWORD dwUserData1,
                                                           DWORD dwUserData2)
{
    //--------------------------------------------------------------------------------
    // sData strFileName  ̹ <APPLOCAL>/profile/ ID/   Ǿִ!
    //--------------------------------------------------------------------------------
    
	if(_HasTransaction(trid) == false)
		return true;

    P_GT_PROFILE_IMAGE_RESULT_DATA sResultData;
    
	//   ̹ ε  óϸ ȴ!
	// ٿε  쿣 OnProfileImageReady  Լ !
	if(bUploadFile)
    {
        // ٸ ׳ !
        if(StringUtil::StrCompare(sData.strUserID, m_pMessengerActor->GetUserID()) != 0)
            return true;
        
        _OnResponseSuccUploadImage(sData);
    }
    else
    {
        _OnResponseSuccDownloadImage(sData);
        
        if(eErrorCode == wasError_None && (eProfileError == piError_None || eProfileError == piError_NoImage))
        {
            SaveCurrentProfileImageCRC(sData.strUserID, sData.dwCRC, sData.eSizeType); 
        }
    }
    
	sResultData.trid = trid;
	sResultData.eProfileError = eProfileError;
	sResultData.eWasError = eErrorCode;
	sResultData.strUserID = sData.strUserID;
	sResultData.dwProfileImageCRC = sData.dwCRC;
    sResultData.strFilePath = sData.strFileName;
    sResultData.eSizeType = sData.eSizeType;
    
	if((sResultData.eWasError == wasError_None) && 
	   (sResultData.eProfileError == piError_None) )
        sResultData.bSuccess = true;
	else
        sResultData.bSuccess = false;
    
    if(bUploadFile)
    {
        CSetMyProfileImageNotifyCommand *pNotify = CSetMyProfileImageNotifyCommand::Create(this, sResultData);
        if(pNotify)
            pNotify->ExecuteAsync();    
    }
    else
    {
        CProfileImageResponseNotifyCommand *pNotify = CProfileImageResponseNotifyCommand::Create(this, sResultData, dwUserData1, dwUserData2, false);
        if(pNotify)
            pNotify->ExecuteAsync();
    }
    
	return true;
}

